(function ($, Drupal) {
  Drupal.behaviors.eventCalendarView = {
    attach: function (context, settings) {
      once('event-calendar', '.mini-calendar-wrapper', context).forEach(function(element) {
        let currentDate = new Date();
        const calendar = $(element);
        const firstDayOfWeek = settings.eventCalendarView.firstDay || 0;

        // General utility functions
        function formatDate(date) {
          if (!(date instanceof Date) || isNaN(date)) {
            console.error('Invalid date object');
            return null;
          }
          const year = date.getFullYear();
          const month = String(date.getMonth() + 1).padStart(2, '0');
          const day = String(date.getDate()).padStart(2, '0');
          return `${year}-${month}-${day}`;
        }

        function updateMonthDisplay(year, month) {
          // Create date for current month
          const date = new Date(year, month, 1);
          
          // Update current month/year display
          $('.current-month', calendar).text(
            date.toLocaleDateString('fr', { 
              month: 'long',
              year: 'numeric'
            })
          );
          
          // Create dates for navigation labels
          const prevMonth = new Date(year, month - 1, 1);
          const nextMonth = new Date(year, month + 1, 1);
          
          // Update navigation button labels
          $('.prev-month', calendar).text(
            prevMonth.toLocaleDateString('fr', { month: 'short' }).toLowerCase()
          );
          $('.next-month', calendar).text(
            nextMonth.toLocaleDateString('fr', { month: 'short' }).toLowerCase()
          );
        }
        

        function filterEvents(selectedDate) {
          $('.event-item, .date-header').hide();
          const selectedEvents = $(`.event-item[data-date="${selectedDate}"]`);
          
          if (selectedEvents.length) {
            selectedEvents.show();
            selectedEvents.prev('.date-header').show();
          } else {
            $('.no-events').show();
          }
          
          $('.day').removeClass('selected');
          $('.day').each(function() {
            const $day = $(this);
            const dayDate = $day.data('date');
            if (dayDate === formatDate(new Date())) {
              $day.addClass('today');
            }
            if (dayDate === selectedDate) {
              $day.addClass('selected');
            }
          });
        }
        
        
        function getWeekNumber(date) {
          const d = new Date(Date.UTC(date.getFullYear(), date.getMonth(), date.getDate()));
          const dayNum = d.getUTCDay() || 7;
          d.setUTCDate(d.getUTCDate() + 4 - dayNum);
          const yearStart = new Date(Date.UTC(d.getUTCFullYear(), 0, 1));
          return Math.ceil((((d - yearStart) / 86400000) + 1) / 7);
        }

        function getDateOfWeek(week, year, firstDayOfWeek) {
          const januaryFirst = new Date(year, 0, 1);
          const firstDayOffset = (januaryFirst.getDay() - firstDayOfWeek + 7) % 7;
          const firstWeekStart = new Date(year, 0, 1 - firstDayOffset);
          const weekStart = new Date(firstWeekStart);
          weekStart.setDate(firstWeekStart.getDate() + (week - 1) * 7);
          return weekStart;
        }

        function selectWeek(weekNumber, year) {
          const firstDayOfWeek = parseInt(settings.eventCalendarView.firstDay);
          
          // Clear ALL previous selections
          $('.week-number, .day').removeClass('selected');
          
          // Calculate week dates
          const weekStart = getDateOfWeek(weekNumber, year, firstDayOfWeek);
          const weekEnd = new Date(weekStart);
          weekEnd.setDate(weekStart.getDate() + 6);
          
          // Select ONLY the exact week number clicked
          $(`.week-number`).filter(function() {
            return parseInt($(this).text()) === weekNumber;
          }).addClass('selected');
          
          // Select corresponding days
          $('.day').each(function() {
            const dayDate = new Date($(this).data('date'));
            if (dayDate >= weekStart && dayDate <= weekEnd) {
              $(this).addClass('selected');
            }
          });
          
          return weekStart;
        }

        function updateNavigationDisplay(month) {
          const today = new Date();
          const currentMonth = today.getMonth();
          
          // Show/hide prev month button
          $('.prev-month').prop('disabled', false);
          
          // Show/hide next month button
          $('.next-month').prop('disabled', false);
          
          // Update today button state
          $('.today').prop('disabled', 
            today.getMonth() === month && 
            today.getFullYear() === currentDate.getFullYear()
          );
        }
        
        

        function calculateEventDuration(event) {
          const startTime = event.find('time[datetime]').attr('datetime');
          const endTime = event.find('time[datetime-end]').attr('datetime-end');
          
          if (!startTime || !endTime) {
            console.error('Missing start or end time from datetime field');
            return 60;
          }
          
          const eventStart = new Date(startTime);
          const eventEnd = new Date(endTime);
          
          return Math.abs(eventEnd - eventStart) / (1000 * 60);
        }

        function calculateEventHeight(durationInMinutes) {
          return (durationInMinutes / 60) * 60; // 60px per hour
        }

        function isDateInWeek(date, weekStart) {
          const weekEnd = new Date(weekStart);
          weekEnd.setDate(weekStart.getDate() + 6);
          return date >= weekStart && date <= weekEnd;
        }

        function checkForEvents(date) {
          const dateString = formatDate(date);
          return $('.event-item[data-date="' + dateString + '"]').length > 0;
        }

        function createEventElement(event, startHour) {
          // Extract date and time from the field content
          const dateField = event.find('.field-date_et_heure_de_rendez_vous').text().trim();
          
          // Parse date and time components
          const [datePart, timePart] = dateField.split(', ');
          const [day, monthStr, year] = datePart.split(' ');
          const [startTime, endTime] = timePart.split(' - ');
          
          // Map French month abbreviations to numbers
          const monthMap = {
            'jan': '01', 'fév': '02', 'mar': '03', 'avr': '04',
            'mai': '05', 'juin': '06', 'juil': '07', 'aoû': '08',
            'sep': '09', 'oct': '10', 'nov': '11', 'déc': '12'
          };
          
          // Format times to HH:mm
          const formatTime = (time) => {
            if (time.includes(':')) return time;
            return `${time.substring(0,2)}:${time.substring(2)}`;
          };
          
          // Create ISO datetime strings
          const month = monthMap[monthStr.toLowerCase()];
          const paddedDay = day.padStart(2, '0');
          const formattedStartTime = formatTime(startTime);
          const formattedEndTime = formatTime(endTime);
          
          const startDateTime = `${year}-${month}-${paddedDay}T${formattedStartTime}:00`;
          const endDateTime = `${year}-${month}-${paddedDay}T${formattedEndTime}:00`;
          
          // Create Date objects for duration calculation
          const eventStart = new Date(startDateTime);
          const eventEnd = new Date(endDateTime);
          
          // Calculate duration and height
          const durationInMinutes = Math.abs(eventEnd - eventStart) / (1000 * 60);
          const heightInPixels = (durationInMinutes / 60) * 60;
          
          // Add time element if not exists
          if (!event.find('time').length) {
            event.append($('<time>', {
              datetime: startDateTime,
              'datetime-end': endDateTime
            }));
          }
          
          // Create event element
          const eventEl = $('<div>', {
            class: 'week-event',
            css: {
              position: 'absolute',
              height: `${heightInPixels}px`,
              left: '4px',
              right: '4px',
              background: '#e8f0fe',
              borderLeft: '3px solid #1a73e8',
              borderRadius: '4px',
              padding: '8px',
              overflow: 'hidden',
              zIndex: 2
            }
          });
          
          // Add event content
          event.find('.field-item').each(function() {
            const content = $(this).html();
            eventEl.append($('<div>', {
              class: `event-field ${$(this).attr('class')}`,
              html: content
            }));
          });
          
          return eventEl;
        }
        
        function positionEventInGrid(event, startHour) {
          const dateStr = event.find('time[datetime]').attr('datetime');
          const eventStart = new Date(dateStr);
          const hours = eventStart.getHours();
          const minutes = eventStart.getMinutes();
          const topPosition = (hours - startHour) * 60 + minutes;
          const durationInMinutes = calculateEventDuration(event);
          const heightInPixels = (durationInMinutes / 60) * 60;
          
          return {
            top: `${topPosition}px`,
            left: '0',
            width: '100%',
            height: `${heightInPixels}px`,
            position: 'absolute'
          };
        }

        function generateCalendar(year, month) {
          const firstDay = new Date(year, month, 1);
          const lastDay = new Date(year, month + 1, 0);
          const firstDayOfWeek = parseInt(settings.eventCalendarView.firstDay);
          
          // Define weekday labels
          let weekdayLabels;
          if (settings.eventCalendarView.weekday_label_language === 'fr') {
            weekdayLabels = ['Di', 'Lu', 'Ma', 'Me', 'Je', 'Ve', 'Sa'];
          } else {
            weekdayLabels = ['S', 'M', 'T', 'W', 'T', 'F', 'S'];
          }
          
          // Rotate labels based on first day setting
          const rotatedLabels = [
            ...weekdayLabels.slice(firstDayOfWeek),
            ...weekdayLabels.slice(0, firstDayOfWeek)
          ];
          
          // Apply weekday labels
          $('.weekdays div').each(function(index) {
            $(this).text(rotatedLabels[index]);
          });
          
          let firstDayWeekday = firstDay.getDay() - firstDayOfWeek;
          if (firstDayWeekday < 0) firstDayWeekday += 7;
          
          const prevMonthDays = new Date(year, month, 0).getDate();
          let days = '';
          let weekNumbers = '';
          let dayCounter = 0;
          
          // Initialize with the correct week number
          let currentWeekNumber = getWeekNumber(new Date(year, month, 1));
          
          // Generate previous month days
          for (let i = firstDayWeekday - 1; i >= 0; i--) {
            const prevDate = new Date(year, month - 1, prevMonthDays - i);
            if (dayCounter % 7 === 0) {
              weekNumbers += `<div class="week-number">${currentWeekNumber}</div>`;
            }
            days += `<div class="day other-month" data-date="${formatDate(prevDate)}">${prevMonthDays - i}</div>`;
            dayCounter++;
          }
          
          // Generate current month days
          const today = new Date();
          for (let day = 1; day <= lastDay.getDate(); day++) {
            currentDate = new Date(year, month, day);
            if (dayCounter % 7 === 0) {
              currentWeekNumber = getWeekNumber(currentDate);
              weekNumbers += `<div class="week-number">${currentWeekNumber}</div>`;
            }
            const isToday = today.getDate() === day && 
                            today.getMonth() === month && 
                            today.getFullYear() === year;
            const hasEvents = checkForEvents(currentDate);
            
            days += `
              <div class="day${isToday ? ' today' : ''}${hasEvents ? ' has-events' : ''}" 
                   data-date="${formatDate(currentDate)}">
                ${day}
              </div>`;
            dayCounter++;
          }
          
          // Generate next month days
          const remainingDays = 42 - (firstDayWeekday + lastDay.getDate());
          for (let day = 1; day <= remainingDays; day++) {
            const nextDate = new Date(year, month + 1, day);
            if (dayCounter % 7 === 0) {
              currentWeekNumber = getWeekNumber(nextDate);
              weekNumbers += `<div class="week-number">${currentWeekNumber}</div>`;
            }
            days += `<div class="day other-month" data-date="${formatDate(nextDate)}">${day}</div>`;
            dayCounter++;
          }
          
          $('.week-numbers').html(weekNumbers);
          $('.days').html(days);
          updateMonthDisplay(year, month);
          updateNavigationDisplay(month);
        }
        
        function generateWeekView(date) {
          if (window.innerWidth <= 768) {
            generateMobileWeekView(date);
          } else {
            const weekStart = new Date(date);
            let dayOfWeek = weekStart.getDay();
            let diff = dayOfWeek - firstDayOfWeek;
            if (diff < 0) diff += 7;
            weekStart.setDate(weekStart.getDate() - diff);
        
            // Get time slot settings and set CSS variables
            const startTime = settings.eventCalendarView.time_slot_start.split(':');
            const endTime = settings.eventCalendarView.time_slot_end.split(':');
            const startHour = parseInt(startTime[0]);
            const endHour = parseInt(endTime[0]);
            document.documentElement.style.setProperty('--start-hour', startHour);
            document.documentElement.style.setProperty('--end-hour', endHour);
        
            // Add week date range header
    const weekEnd = new Date(weekStart);
    weekEnd.setDate(weekStart.getDate() + 6);
    const weekDateRange = `${weekStart.toLocaleDateString('fr', { 
      day: 'numeric',
      month: 'long'
    })} - ${weekEnd.toLocaleDateString('fr', { 
      day: 'numeric',
      month: 'long',
      year: 'numeric'
    })}`;
    
    $('.week-view').empty().html(`
      <div class="week-date-range">${weekDateRange}</div>
      <div class="week-header">
        <div class="time-column"></div>
        <div class="weekday-headers"></div>
      </div>
      <div class="week-grid">
        <div class="time-slots"></div>
        <div class="event-grid"></div>
      </div>
    `);
        
            // Setup weekday headers
            const weekDays = $('.weekday-headers');
            weekDays.empty();
            let weekdayLabels = settings.eventCalendarView.weekday_label_language === 'fr' 
              ? ['L', 'M', 'M', 'J', 'V', 'S', 'D']
              : ['M', 'T', 'W', 'T', 'F', 'S', 'S'];
        
            if (firstDayOfWeek === 0) {
              weekdayLabels.unshift(weekdayLabels.pop());
            }
        
            // Generate week headers
            for (let i = 0; i < 7; i++) {
              const currentDate = new Date(weekStart);
              currentDate.setDate(weekStart.getDate() + i);
              const formattedDate = formatDate(currentDate);
              weekDays.append(`
                <div class="weekday-header" data-date="${formattedDate}">
                  <div class="weekday">${weekdayLabels[i]}</div>
                  <div class="date">${currentDate.getDate()}</div>
                </div>
              `);
            }
        
            // Generate time slots
            const timeSlots = $('.time-slots');
            timeSlots.empty();
            for (let hour = startHour; hour <= endHour; hour++) {
              timeSlots.append(`<div class="time-slot">${hour}:00</div>`);
            }
        
            // Setup event grid
            const eventGrid = $('.event-grid');
            eventGrid.empty();
            for (let i = 0; i < 7; i++) {
              eventGrid.append('<div class="day-column"></div>');
            }
        
            // Position events
            $('.event-item').each(function() {
              const eventDate = new Date($(this).data('date'));
              if (isDateInWeek(eventDate, weekStart)) {
                const eventEl = createEventElement($(this), startHour);
                if (eventEl) {
                  const position = positionEventInGrid($(this), startHour);
                  eventEl.css({
                    position: 'absolute',
                    top: position.top,
                    left: position.left,
                    width: position.width,
                    height: position.height,
                    zIndex: 2
                  });
                  const dayIndex = (eventDate.getDate() - weekStart.getDate());
                  $(`.day-column:eq(${dayIndex})`).append(eventEl);
                }
              }
            });
          }
        }

        function generateMobileWeekView(date) {
          const weekStart = new Date(date);
          let dayOfWeek = weekStart.getDay();
          let diff = dayOfWeek - firstDayOfWeek;
          if (diff < 0) diff += 7;
          weekStart.setDate(weekStart.getDate() - diff);
        
          $('.week-view').empty();
        
          // Generate mobile day containers for each day of the week
          for (let i = 0; i < 7; i++) {
            const currentDate = new Date(weekStart);
            currentDate.setDate(weekStart.getDate() + i);
            
            const container = $('<div>', {
              class: 'mobile-day-container'
            });
        
            // Add date header
            const formattedDate = currentDate.toLocaleDateString('fr', {
              weekday: 'long',
              day: 'numeric',
              month: 'long'
            });
            
            container.append(`
              <div class="date-header">${formattedDate}</div>
              <div class="day-grid">
                <div class="time-slots"></div>
                <div class="events-column"></div>
              </div>
            `);
        
            // Add time slots
            const startHour = parseInt(settings.eventCalendarView.time_slot_start);
            const endHour = parseInt(settings.eventCalendarView.time_slot_end);
            
            const timeSlots = container.find('.time-slots');
            for (let hour = startHour; hour <= endHour; hour++) {
              timeSlots.append(`
                <div class="time-slot">${String(hour).padStart(2, '0')}:00</div>
              `);
            }
        
            // Add events for this day
            const eventsColumn = container.find('.events-column');
            $('.event-item').each(function() {
              const eventDate = new Date($(this).data('date'));
              if (formatDate(eventDate) === formatDate(currentDate)) {
                const eventEl = createEventElement($(this), startHour);
                if (eventEl) {
                  const position = positionEventInGrid($(this), startHour);
                  eventEl.css(position);
                  eventsColumn.append(eventEl);
                }
              }
            });
        
            $('.week-view').append(container);
          }
        }

        function initMobileView() {
          if (window.innerWidth <= 768) {
            // Remove any existing click handlers first
            $('.calendar-header').off('click');
            
            // Add click handler for mobile view
            $('.calendar-header').on('click', function() {
              $('.mini-calendar-wrapper').toggleClass('collapsed');
              
              // Adjust week view height when calendar is collapsed/expanded
              if ($('.week-view').is(':visible')) {
                generateWeekView(currentDate);
              }
            });
          } else {
            // Remove collapse/expand classes and click handler for desktop view
            $('.mini-calendar-wrapper').removeClass('collapsed');
            $('.calendar-header').off('click');
          }
        }
        
        function generateDayView(date) {
          const startHour = parseInt(settings.eventCalendarView.time_slot_start);
          const endHour = parseInt(settings.eventCalendarView.time_slot_end);
          
          document.documentElement.style.setProperty('--start-hour', startHour);
          document.documentElement.style.setProperty('--end-hour', endHour);
          
          // Combine day name and date in a single header
          const formattedDate = date.toLocaleDateString('fr', {
            weekday: 'long',
            day: 'numeric',
            month: 'long',
            year: 'numeric'
          });
          
          // Create day view structure
          $('.day-view').empty().html(`
            <div class="day-header">${formattedDate}</div>
            <div class="day-grid">
              <div class="time-slots"></div>
              <div class="events-column"></div>
            </div>
          `);
          
          // Generate time slots
          const timeSlots = $('.day-view .time-slots');
          for (let hour = startHour; hour <= endHour; hour++) {
            timeSlots.append(`
              <div class="time-slot">
                ${String(hour).padStart(2, '0')}:00
              </div>
            `);
          }
          
          const eventsColumn = $('.day-view .events-column');
          
          $('.event-item').each(function() {
            const eventDateStr = $(this).data('date');
            const selectedDateStr = formatDate(date);
            if (eventDateStr === selectedDateStr) {
              const eventEl = createEventElement($(this), startHour);
              if (eventEl) {
                const position = positionEventInGrid($(this), startHour);
                eventEl.css(position);
                eventsColumn.append(eventEl);
              }
            }
          });
        }
        
        
        
        // Navigation handlers
        $('.prev-month', calendar).on('click', function() {
  // Store current month and year before modification
  let newMonth = currentDate.getMonth() - 1;
  let newYear = currentDate.getFullYear();
  
  // Handle year transition when going back from January
  if (newMonth < 0) {
    newMonth = 11;
    newYear--;
  }
  
  // Update currentDate
  currentDate = new Date(newYear, newMonth, 1);
  
  // Generate calendar and update display
  generateCalendar(newYear, newMonth);
  updateMonthDisplay(newYear, newMonth);
});

$('.next-month', calendar).on('click', function() {
  // Store current month and year before modification
  let newMonth = currentDate.getMonth() + 1;
  let newYear = currentDate.getFullYear();
  
  // Handle year transition when going forward from December
  if (newMonth > 11) {
    newMonth = 0;
    newYear++;
  }
  
  // Update currentDate
  currentDate = new Date(newYear, newMonth, 1);
  
  // Generate calendar and update display
  generateCalendar(newYear, newMonth);
  updateMonthDisplay(newYear, newMonth);
});
        
        $('.today', calendar).on('click', function() {
          const today = new Date();
          currentDate = today;
          generateCalendar(today.getFullYear(), today.getMonth());
          filterEvents(formatDate(today));
        });
        
        // Week number click handler
        $(document).on('click', '.week-number', function() {
          const clickedWeekNumber = parseInt($(this).text());
          const currentYear = currentDate.getFullYear();
          
          // Update current date to the clicked week
          const weekStart = getDateOfWeek(clickedWeekNumber, currentYear, firstDayOfWeek);
          currentDate = new Date(weekStart);
          
          // Clear and update selections
          $('.week-number, .day').removeClass('selected');
          $(this).addClass('selected');
          
          // Generate week view with the new date
          generateWeekView(weekStart);
          selectWeek(clickedWeekNumber, currentYear);
        });
        
        // View switch handler
        $('.view-switch-button').on('click', function() {
  const viewType = $(this).data('view');
  $('.view-switch-button').removeClass('active');
  $(this).addClass('active');
  
  $('.events-list, .week-view, .day-view').hide();
  
  if (viewType === 'list') {
    $('.events-list').show();
    generateCalendar(currentDate.getFullYear(), currentDate.getMonth());
  } else if (viewType === 'week') {
    $('.week-view').show();
    // Always use today's date when switching to week view
    const today = new Date();
    const currentWeek = getWeekNumber(today);
    const weekStart = getDateOfWeek(currentWeek, today.getFullYear(), firstDayOfWeek);
    
    // Update currentDate to today
    currentDate = today;
    
    // Generate week view and select current week
    generateWeekView(weekStart);
    selectWeek(currentWeek, today.getFullYear());
  } else if (viewType === 'day') {
    $('.day-view').show();
    $('.week-number, .day').removeClass('selected');
    currentDate = new Date();
    $('.day').removeClass('today');
    $(`.day[data-date="${formatDate(currentDate)}"]`).addClass('today selected');
    generateDayView(currentDate);
  }
});
        
        // Day click handler
        $(calendar).on('click', '.day', function() {
          const date = $(this).data('date');
          if (date) {
            const selectedDate = new Date(date);
            if ($('.week-view').is(':visible')) {
              generateWeekView(selectedDate);
            } else if ($('.day-view').is(':visible')) {
              generateDayView(selectedDate);
            } else {
              filterEvents(formatDate(selectedDate));
            }
            $('.day').removeClass('selected');
            $(this).addClass('selected');
          }
        });
        
        // Window resize handler
        $(window).on('resize', function() {
          if ($('.week-view').is(':visible')) {
            if (window.innerWidth <= 768) {
              generateMobileWeekView(currentDate);
            } else {
              $('.week-view').empty();
              generateWeekView(currentDate);
            }
          }
          initMobileView();
        });
        
        // Initialize calendar
        generateCalendar(currentDate.getFullYear(), currentDate.getMonth());
        initMobileView();
        $(window).on('resize', initMobileView);
        
        });
        }
        };
        })(jQuery, Drupal);



        


        


        
        
