<?php

namespace Drupal\event_calendar_view\Plugin\views\style;

use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\style\StylePluginBase;

/**
 * @ViewsStyle(
 *   id = "event_calendar_view",
 *   title = @Translation("Event Calendar View"),
 *   help = @Translation("Display content in an interactive calendar layout."),
 *   theme = "views_view_event_calendar",
 *   display_types = {"normal"}
 * )
 */
class EventCalendarView extends StylePluginBase {
  protected $usesRowPlugin = TRUE;
  protected $usesFields = TRUE;

  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['date_field'] = ['default' => ''];
    $options['first_day'] = ['default' => 0];
    $options['weekday_label_language'] = ['default' => 'en'];
    $options['time_slot_start'] = ['default' => '08:00'];
    $options['time_slot_end'] = ['default' => '20:00'];
    return $options;
  }

  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
    
    $field_options = [];
    foreach ($this->displayHandler->getHandlers('field') as $field_id => $field) {
      $field_options[$field_id] = $field->adminLabel();
    }

    $form['date_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Date Field'),
      '#options' => $field_options,
      '#required' => TRUE,
      '#default_value' => $this->options['date_field'],
    ];

    $form['first_day'] = [
      '#type' => 'select',
      '#title' => $this->t('First day of week'),
      '#options' => [
        0 => $this->t('Sunday'),
        1 => $this->t('Monday'),
      ],
      '#default_value' => $this->options['first_day'],
    ];

    $form['weekday_label_language'] = [
      '#type' => 'select',
      '#title' => $this->t('Weekday label language'),
      '#options' => [
        'en' => $this->t('English'),
        'fr' => $this->t('French'),
      ],
      '#default_value' => $this->options['weekday_label_language'],
    ];

    // Generate hour options for time slots
    $hours = [];
    for ($i = 0; $i < 24; $i++) {
      $hour = str_pad($i, 2, '0', STR_PAD_LEFT) . ':00';
      $hours[$hour] = $hour;
    }

    $form['time_slot_start'] = [
      '#type' => 'select',
      '#title' => $this->t('Week View Start Time'),
      '#options' => $hours,
      '#default_value' => $this->options['time_slot_start'],
      '#required' => TRUE,
    ];
    
    $form['time_slot_end'] = [
      '#type' => 'select',
      '#title' => $this->t('Week View End Time'),
      '#options' => $hours,
      '#default_value' => $this->options['time_slot_end'],
      '#required' => TRUE,
    ];
  }

  public function render() {
    $build = [];
    $build['#theme'] = $this->themeFunctions();
    $build['#view'] = $this->view;
    $build['#options'] = $this->options;
    $build['#rows'] = [];

    foreach ($this->view->result as $row_index => $row) {
      $row_fields = [];
      foreach ($this->view->field as $field_id => $field) {
        if (empty($field->options['exclude'])) {
          $row_fields[$field_id] = $this->getField($row_index, $field_id);
        }
      }

      $date = $this->getFieldValue($row, $this->options['date_field']);
      if ($date) {
        $build['#rows'][] = [
          'date' => $date,
          'content' => $row_fields,
          'raw' => $row,
        ];
      }
    }

    $build['#attached']['library'][] = 'event_calendar_view/calendar-lib';
    $build['#attached']['drupalSettings']['eventCalendarView'] = [
      'dateField' => $this->options['date_field'],
      'firstDay' => $this->options['first_day'],
      'weekday_label_language' => $this->options['weekday_label_language'],
      'time_slot_start' => $this->options['time_slot_start'],
      'time_slot_end' => $this->options['time_slot_end'],
    ];
    
    return $build;
  }

  public function getFieldValue($row, $field_id) {
    if (isset($this->view->field[$field_id])) {
      $field = $this->view->field[$field_id];
      if (!empty($field->options['exclude'])) {
        return NULL;
      }
      return $field->getValue($row);
    }
    return NULL;
  }
}
